import { Widget } from "../model/Widget";
import { WidgetPackage } from "../model/WidgetPackage";
import { Channel } from "./Channel";
import { UrlUtils } from "../utils/UrlUtils";
import { BroadcastEvent } from "../model/event/BroadcastEvent";
import { BroadcastApi } from "./BroadcastApi";
import localforage from "localforage";
import { BaseApi } from "./BaseApi";
var WidgetApiEvent;
(function (WidgetApiEvent) {
    WidgetApiEvent["DATA_CHANGED"] = "event::cn.widgetjs.core.widget.data0changed";
    WidgetApiEvent["EDIT_DESKTOP_WIDGETS"] = "event::cn.widgetjs.core.widget.desktop.edit";
})(WidgetApiEvent || (WidgetApiEvent = {}));
class WidgetApiImpl extends BaseApi {
    constructor() {
        super(...arguments);
        this.stores = new Map();
    }
    getChannel() {
        return Channel.WIDGET;
    }
    async registerWidgets(widgets) {
        return this.invokeMethod("registerWidgets", JSON.stringify(widgets));
    }
    async registerWidgetPackage(widgetPackage) {
        return this.invokeMethod("registerWidgetPackage", JSON.stringify(widgetPackage));
    }
    async getWidgets() {
        const data = await this.invokeMethod('getWidgets');
        const widgets = [];
        if (data) {
            for (const item of data) {
                widgets.push(Widget.parseObject(item));
            }
        }
        return widgets;
    }
    async getWidgetPackages() {
        return await this.invokeMethod('getWidgetPackages');
    }
    /**
     *
     * @param name package name
     */
    async getWidget(name) {
        return Widget.parseObject(await this.invokeMethod('getWidget', name));
    }
    /**
     *
     * @param name package name
     */
    async getWidgetPackage(name) {
        return WidgetPackage.parseObject(await this.invokeMethod('getWidgetPackage', name));
    }
    /**
     * 获取组件配置地址
     * @param widgetName
     */
    async getWidgetConfigUrl(widgetName, widgetParams) {
        const widget = await this.getWidget(widgetName);
        if (!widget)
            return null;
        const configRoute = widget.getConfigRoute();
        if (!configRoute)
            return null;
        const widgetPackage = await this.getWidgetPackage(widget.packageName);
        if (!widgetPackage)
            return null;
        return UrlUtils.getWidgetUrl(configRoute.url, widgetPackage, widgetParams);
    }
    async getWidgetUrl(widgetName, widgetParams) {
        const widget = await this.getWidget(widgetName);
        if (!widget)
            return null;
        const indexRoute = widget.getIndexRoute();
        const widgetPackage = await this.getWidgetPackage(widget.packageName);
        if (!widgetPackage)
            return null;
        return UrlUtils.getWidgetUrl(indexRoute.url, widgetPackage, widgetParams);
    }
    async getWidgetPackageIndexUrl(packageName, hash) {
        const widgetPackage = await this.getWidgetPackage(packageName);
        if (!widgetPackage)
            return null;
        return widgetPackage.getIndexUrl(hash);
    }
    async getWidgetPackageUrl(packageName) {
        const widgetPackage = await this.getWidgetPackage(packageName);
        if (!widgetPackage)
            return null;
        return widgetPackage.url;
    }
    /**
     * 通过组件名保存组件信息，通常用于存储可以在同类组件中共用的数据
     * @param data
     * @param options
     */
    async saveDataByName(data, options = { sendBroadcast: true }) {
        const store = this.getStore(data.name);
        const json = JSON.stringify(data);
        const result = await store.setItem(data.name, json);
        if (options.sendBroadcast) {
            const broadcastEvent = new BroadcastEvent({
                event: WidgetApiEvent.DATA_CHANGED,
                payload: {
                    name: data.name,
                    id: options.id,
                    json
                }
            });
            await BroadcastApi.send(broadcastEvent);
        }
        return result;
    }
    /**
     * 获取组件 LocalForage 存储实例
     * @param name
     */
    getStore(name) {
        if (this.stores.has(name)) {
            return this.stores.get(name);
        }
        const store = localforage.createInstance({ name: name });
        this.stores.set(name, store);
        return store;
    }
    addWidget(widgetName, hostMode) {
        return this.invokeMethod('addWidget', widgetName, hostMode);
    }
}
const WidgetApi = new WidgetApiImpl();
export { WidgetApi, WidgetApiEvent };
